<?php

//
// innovaphone fax mailbox polling PHP script
// copyright (c) innovaphone 2012 - 2015
//

$path=dirname(__FILE__);
require_once("$path/../include.php");
require_once("classes/translator.class.php");
require_once("classes/inputpage.class.php");
require_once("classes/mail/innovaphone.mailbox.class.php");
require_once("$path/../app-classes/innovaphone.fax.config.class.php");
require_once("$path/../app-classes/innovaphone.fax.shm.class.php");
require_once("$path/../app-classes/innovaphone.fax.database.class.php");

$version = file_get_contents("$path/../version");

declare(ticks = 1);
?>

innovaphone fax mailbox polling PHP script
copyright (c) innovaphone 2012 - 2015

<?php

print "Version " . $version . "\n\n";

$help = false;
foreach($argv as $arg) {
    if(in_array($arg, array('--help', '-help', '-h', '-?'))) {
        $help = true;
    }
}

if(!is_array($_GET) && ($argc < 2 || $help)) {
?>
    Usage:
  <?php echo $argv[0]; ?> <id>

<id> is the configuration database id specific for this server instance.

With the --help, -help, -h, or -? options, you can get this help.

<?php
exit(1);
} 

$configurationId        = null;
if($argc >= 1) {
    $configurationId    = intval($argv[1]);
}
else if(is_array($_GET)) {
    $configurationId    = intval($_GET['id']);
}


setlocale(LC_CTYPE, "en_US.UTF-8");

function getLogDate() {
    $now = new DateTime();
    return date('Ymd-His') . "  -  ";
}

function sig_handler($nsig)
{
    print "\n";
    print getLogDate() . "terminated($nsig).\n\n";
    exit;
}

pcntl_signal(SIGTERM, "sig_handler", false);
pcntl_signal(SIGINT, "sig_handler", false);
pcntl_signal(SIGHUP, "sig_handler", false);


$sleepTimeout = 0;  
function mailsleep() {
    global $sleepTimeout;
    if($sleepTimeout > 0) {
        sleep($sleepTimeout);
    }
    if($sleepTimeout == 0) {
        $sleepTimeout = 5;          // start with 5 seconds
    }
    else {
        if($sleepTimeout < 60) {    // increase until 60 seconds
            $sleepTimeout++;        
        }
    }
}

function doOutgoingConvert(&$cfg, &$mailbox, &$faxDatabase, &$faxShm, $configurationId, &$startMax) {
    $count = 0;
    $mailInfos = $mailbox->getMailInfos();
    if($mailInfos === false) {
        return false;
    }
       
    foreach($mailInfos as $num => $mailInfo) {
        if($faxShm->read() === false) {
            return false;
        }
        $found = $faxShm->findJobMessageId($mailInfo->getMessageId()) != null;
            
        if(!$found) {
            $jobs   = $faxDatabase->getFaxJobs(array('message_id' => $mailInfo->getMessageId()));
                
            if($jobs && is_array($jobs)) {
                foreach($jobs as $job) {
                    if($job->getProgress() == 'queued' || $job->getProgress() == 'closed') {
                        $found = true;
                        break;
                    }
                }
            }
        }
        if(!$found) {
            $count++;
            print getLogDate() . "Converting documents of received mail from '" . $mailInfo->getSender()->mailbox . "'... ";
            global $path;
            $script     = $path . DIRECTORY_SEPARATOR . "innovaphone.fax.convert.out.php";
            $messageId  = "";
            $m = $mailInfo->getMessageId();
            for($i = 0; $i < strlen($m); $i++) {
                if(('a' <= $m[$i] && $m[$i] <= 'z') ||
                    ('A' <= $m[$i] && $m[$i] <= 'Z') ||
                    ('0' <= $m[$i] && $m[$i] <= '9'))
                {
                    $messageId .= $m[$i];
                }
            }
            $log        = ">> ".FaxConfig::logPath."/" . $configurationId . '-convert-out-' . $messageId . '.log 2>&1';
            $messageId  = escapeshellarg($mailInfo->getMessageId());
            $exec       = "/usr/bin/php5 $script $configurationId $messageId $log";
            $outputs    = array();
            $ret        = false;
            //print $exec . "\n";
            exec($exec, $outputs, $ret);
            print "\n" . getLogDate() . "done. \n";
                
            if(!startOutgoingCalls($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId, $startMax)) return false;
            if(!doOutgoingConfirm($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId)) return false;
            if(!doIncomingConvert($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId)) return false;
            if(!doIncomingForward($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId)) return false;
        }
        $mailbox->setFlag($mailInfo->getMessageId(), true);
    }
    return $count;
}

function startOutgoingCalls(&$cfg, &$mailbox, &$faxDatabase, &$faxShm, $configurationId, &$startMax) {
    $queuedJobs = $faxDatabase->getFaxJobs(array('direction' => 'outgoing', 'progress' => 'queued'));
    if($queuedJobs === false) {
        return false;
    }
       
    foreach($queuedJobs as &$faxJob) {
        $now = new DateTime();
        if($startMax <= 0) {
            break;
        }
        if($faxShm->read() === false) {
            return false;
        }
        if(!$faxShm->getJob($faxJob->getGuid()) &&
            (!$faxJob->getDelayed() || ($faxJob->getDelayed() <= $now)) &&
            $faxJob->getResult() != 'success' &&
            $faxJob->getResult() != 'error')
        {
            $startMax--;
            print getLogDate() . "Send fax from '" . $faxJob->getUserH323() . "' to '" . $faxJob->getRemoteNumber() . "'" . ($startMax <= 0 ? ", maximum limit" : "") . ".\n";
            global $path;
            $script = $path . DIRECTORY_SEPARATOR . "innovaphone.fax.call.out.php";
            $log    = ">> ".FaxConfig::logPath."/" . $configurationId . '-call-out-' . $faxJob->getGuid() . '.log 2>&1';
            $exec   = "/usr/bin/php5 $script $configurationId \"{$faxJob->getGuid()}\" $log &";
            //print $exec . "\n";
            exec($exec);
        }
    }
    return true;
}   
        
function doOutgoingConfirm(&$cfg, &$mailbox, &$faxDatabase, &$faxShm, $configurationId) {
    $queuedJobs = $faxDatabase->getFaxJobs(array('direction' => 'outgoing', 'progress' => 'queued'));
    if($queuedJobs === false) {
        return false;
    }
        
    foreach($queuedJobs as &$faxJob) {
        $now = new DateTime();
        if($faxJob->getResult() == 'success' ||
            $faxJob->getResult() == 'error')
        {
            $files              = array();
            $confirmationFile   = null;
            if($faxJob->getResult() == 'success' &&
                $cfg->faxConfirmationWithDoc)
            {
                print getLogDate() . "Converting sent fax document... ";

                $fileWebdav         = $faxJob->getFileName();
                $fileData           = pathinfo($fileWebdav);
                $fileName           = $fileData['filename'];
                $filePath           = FaxConfig::documentPath;
                $fileWebdavLocal    = FaxConfig::webdavPath . "/" . $fileData['basename'];
                //$fileConvert        = $filePath . "/" . $fileData['basename'];
                $fileConvert        = $fileWebdavLocal;
                $fileTiff           = $filePath . "/" . $fileName . '.tif';
                $fileConverted      = $filePath . "/" . $fileName . '.pdf';


                $exec       = "sfftobmp -t $fileConvert -o $fileTiff 2>&1";
                $outputs    = array();
                $ret        = false;

                exec($exec, $outputs, $ret);

                if($ret) {
                    print "failed.\n";
                    $output     = "";
                    foreach($outputs as &$row) {
                        $output .= $row . "\n";
                    }
                    print getLogDate() . "Sent fax document '$fileWebdav' could not be converted into TIFF. Exit code $ret.\nOutput:\n$output\n";
                }
                else {
                    $creator    = "\"innovaphone Faxserver\"";
                    $exec       = "tiff2pdf -pA4 -c $creator $fileTiff > $fileConverted";
                    $outputs    = array();
                    $ret        = false;

                    exec($exec, $outputs, $ret);

                    if($ret) {
                        print "failed.\n";
                        $output     = "";
                        foreach($outputs as &$row) {
                            $output .= $row . "\n";
                        }
                        print getLogDate() . "Sent fax document '$fileWebdav' could not be converted into PDF. Exit code $ret.\nOutput:\n$output\n";
                    }
                    else {
                        $confirmationFile   = $fileConverted;
                        $number             = $faxJob->getRemoteNumber();
                        $fileDate           = $faxJob->getCallTimestamp() ? $faxJob->getCallTimestamp()->format("-Ymd-Hi") : "";
                        $files["fax-confirmation-$number$fileDate.pdf"] = $confirmationFile;
                            
                        $exec       = "rm -f $fileTiff 2>&1";
                        $outputs    = array();
                        $ret        = false;
                        exec($exec, $outputs, $ret);
                        print "done.\n";
                    }                        
                }
            }
                
            if($faxJob->getUserLanguage() != "") {
                tl::change($faxJob->getUserLanguage());
            }
            $subject                        = "";
            $message                        = "";
            $info                           = array();
            $info['<result>']               = $faxJob->getResult();
            $info['<user-name>']            = $faxJob->getUserName();
            $info['<user-h323>']            = $faxJob->getUserH323();
            $info['<user-number>']          = $faxJob->getUserNumber();
            $info['<remote-number>']        = $faxJob->getRemoteNumber();
            $info['<remote-subscriber-id>'] = $faxJob->getRemoteSubscriberId();
            $info['<calls>']                = $faxJob->getCalls();
            $info['<pages>']                = $faxJob->getPages();
            $info['<duration>']             = $faxJob->getDuration() ? $faxJob->getDuration()->format('%i:%S') : "-";
            $info['<error-cause>']          = $faxJob->getErrorCause();
            $info['<error-description>']    = $faxJob->getErrorDescription();
            $info['<date>']                 = $faxJob->getCallTimestamp() ? InputPageDateField::translateDate(($faxJob->getCallTimestamp()->getTimestamp())) : "-";
            $info['<time>']                 = $faxJob->getCallTimestamp() ? $faxJob->getCallTimestamp()->format("H:i:s") : "-";
            $info['<subject>']              = "";
            $info['<SAP-code>']	            = "9007";
                
            switch($faxJob->getResult()) {
                case 'success':
                    $subject = $cfg->getString(FaxServerStrings::msgSubjectDeliverSuccess, $faxJob->getUserLanguage());
                    $message = $cfg->getString(FaxServerStrings::msgDeliverSuccess, $faxJob->getUserLanguage());
                    $info['<SAP-code>']	= "0000";
                    break;

                case 'aborted':
                case 'error':
                    $subject = $cfg->getString(FaxServerStrings::msgSubjectDeliverError, $faxJob->getUserLanguage());
                    if($faxJob->getErrorCause() == "response timeout") {
                        $message = $cfg->getString(FaxServerStrings::msgDeliverErrorTimeout, $faxJob->getUserLanguage());
                        $info['<SAP-code>']	= "9003";
                    }
                    else if($faxJob->getErrorCause() == "no FAX device") {
                        $message = $cfg->getString(FaxServerStrings::msgDeliverErrorNoFaxDevice, $faxJob->getUserLanguage());
                        $info['<SAP-code>']	= "5001";
                    }
                    else if($faxJob->getErrorCause() == "busy") {
                        $message = $cfg->getString(FaxServerStrings::msgDeliverErrorBusy, $faxJob->getUserLanguage());
                        $info['<SAP-code>']	= "9002";
                    }
                    else if($faxJob->getErrorCause() == "canceled") {
                        $message = $cfg->getString(FaxServerStrings::msgDeliverErrorCanceled, $faxJob->getUserLanguage());
                        $info['<SAP-code>']	= "9003";
                    }
                    else {
                        $message = $cfg->getString(FaxServerStrings::msgDeliverError, $faxJob->getUserLanguage());
                        if($faxJob->getErrorCause() == "call disconnected") {
                            $info['<SAP-code>']	= "9001";
                        }
                        else if($faxJob->getErrorCause() == "no channnel available") {
                            $info['<SAP-code>']	= "9003";
                        }
                        else if($faxJob->getErrorCause() == "by user") {
                            $info['<SAP-code>']	= "5001";
                        }
                        else if($faxJob->getErrorCause() == "timeout") {
                            $info['<SAP-code>']	= "9006";
                        }
                        else if($faxJob->getErrorCause() == "bad-connection") {
                            $info['<SAP-code>']	= "9000";
                        }
                        else if($faxJob->getErrorCause() == "by-remote") {
                            $info['<SAP-code>']	= "9008";
                        }
                        else if($faxJob->getErrorCause() == "uui") {
                            $info['<SAP-code>']	= "6020";
                        }
                        else if($faxJob->getErrorCause() == "file-open") {
                            $info['<SAP-code>']	= "6019";
                        }
                        else if($faxJob->getErrorCause() == "file-read") {
                            $info['<SAP-code>']	= "6019";
                        }
                        else if($faxJob->getErrorCause() == "file-write") {
                            $info['<SAP-code>']	= "6019";
                        }
                        else if($faxJob->getErrorCause() == "file-type") {
                            $info['<SAP-code>']	= "6019";
                        }
                        else if($faxJob->getErrorCause() == "remote") {
                            $info['<SAP-code>']	= "9010";
                        }
                        else if($faxJob->getErrorCause() == "protocol") {
                            $info['<SAP-code>']	= "9006";
                        }
                        else if($faxJob->getErrorCause() == "call aborted") {
                            $info['<SAP-code>']	= "9003";
                        }
                        else {
                            $info['<SAP-code>']	= "9001";
                        }
                    }
                    break;
                    
                default:
                    $message .= "Unknown result state('" . $faxJob->getResult() . "').\n";
            }
                
            $to             = '"' . mb_encode_mimeheader($faxJob->getUserName(), "UTF-8", "Q", "") . '" <' . $faxJob->getUserH323() . '>';
            $from           = imap_rfc822_write_address("faxserver", $cfg->faxServerDomain, $cfg->name);
            $mail           = $mailbox->getMail($faxJob->getMessageId());
            if($mail) {
                $info['<subject>'] = $mail->getSubject();
                $to                = imap_rfc822_write_address($mail->getSender()->mailbox,
                                                                $mail->getSender()->host,
                                                                isset($mail->getSender()->personal) ? $mail->getSender()->personal : "");
            }
                
            if($cfg->mailSAPCode) {
                $message .= "\n[<SAP-code>]\n";    
            }
                
            $subject = strtr($subject, $info);
            $message = strtr($message, $info);
                
            print getLogDate() . "Sending status to " . ($mail ? ($mail->getSenderMailbox() . '@' . $mail->getSender()->host) : $faxJob->getUserH323()) . "... ";

            $delivered = $mailbox->sendMail($to, $from, $subject, $message, $files, $mail ? $mail->getMessageId() : null);
            if($delivered) {
                print "done.\n";
                    
                if($mail) {
                    $countUnclosedJobs = $faxDatabase->getCountUnclosedFaxJobs($faxJob->getMessageId());
                        
                    if($countUnclosedJobs === false || $countUnclosedJobs <= 1) {
                        print getLogDate() . "Deleting email... ";

                        if(!$mailbox->deleteMail($faxJob->getMessageId())) {
                            print "failed.\n";
                        }
                        else {
                            print "done.\n";
                        }
                            
                        print getLogDate() . "Deleting sent file... ";
                        $fileWebdav         = $faxJob->getFileName();
                        $fileData           = pathinfo($fileWebdav);
                        $fileWebdavLocal    = FaxConfig::webdavPath . "/" . $fileData['basename'];
                        $fileName   = escapeshellarg($fileWebdavLocal);
                        $exec       = "rm -f $fileName 2>&1";
                        $outputs    = array();
                        $ret        = false;
                        exec($exec, $outputs, $ret);
                        if($ret) {
                            print "failed.\n";
                        }
                        else {
                            if($confirmationFile != "") {                        
                                print "deleting confirmation file... ";
                                $fileName   = escapeshellarg($confirmationFile);
                                $exec       = "rm -f $fileName 2>&1";
                                $outputs    = array();
                                $ret        = false;
                                exec($exec, $outputs, $ret);
                                if($ret) {
                                    print "failed.\n";
                                }
                                else {
                                    print "done.\n";
                                }
                            }    
                            else {
                                print "done.\n";
                            }
                        }    
                    }
                }
                    
                print getLogDate() . "Updating database... ";

                $faxJob->setProgress('closed');
                $faxJob = $faxDatabase->updateFaxJob($faxJob);
                if($faxJob) {
                    print "done.\n";
                }
                else {
                    print "failed.\n";
                }
            }
            else {
                print "failed.\n";
            }                    
        }
    }
    return true;
}

function doIncomingConvert(&$cfg, &$mailbox, &$faxDatabase, &$faxShm, $configurationId) {
    $queuedJobs = $faxDatabase->getFaxJobs(array('direction' => 'incoming', 'progress' => 'queued'));
    if($queuedJobs === false) {
        return false;
    }

    if($faxShm->read() === false) {
        return false;
    }
    
    foreach($queuedJobs as &$faxJob) {
        if(!$faxShm->getJob($faxJob->getGuid())) {
            print getLogDate() . "Converting received fax from '" . $faxJob->getRemoteNumber() . "' to '" . $faxJob->getUserH323() . "'... ";
            global $path;
            $script     = $path . DIRECTORY_SEPARATOR . "innovaphone.fax.convert.in.php";
            $log        = ">> ".FaxConfig::logPath."/" . $configurationId . '-convert-in-' . $faxJob->getGuid() . '.log 2>&1';
            $exec       = "/usr/bin/php5 $script $configurationId \"{$faxJob->getGuid()}\" $log";
            $outputs    = array();
            $ret        = false;
            exec($exec, $outputs, $ret);
            print "done.\n";
        }
    }
    return true;
}


function doIncomingForward(&$cfg, &$mailbox, &$faxDatabase, &$faxShm, $configurationId) {
    $convertedJobs = $faxDatabase->getFaxJobs(array('direction' => 'incoming', 'progress' => 'converted'));
    if($convertedJobs === false) {
        return false;
    }

    if($faxShm->read() === false) {
        return false;
    }

    foreach($convertedJobs as &$faxJob) {
        if(!$faxShm->getJob($faxJob->getGuid())) {
            print getLogDate() . "Forwarding received fax from '" . $faxJob->getRemoteNumber() . "' to '" . $faxJob->getUserH323() . "'... ";
            global $path;
            $script     = $path . DIRECTORY_SEPARATOR . "innovaphone.fax.mail.forward.php";
            $log        = ">> ".FaxConfig::logPath."/" . $configurationId . '-mail-forward-' . $faxJob->getGuid() . '.log 2>&1';
            $exec       = "/usr/bin/php5 $script $configurationId \"{$faxJob->getGuid()}\" $log";
            $outputs    = array();
            $ret        = false;
            exec($exec, $outputs, $ret);
            print "done.\n";
        }
    }
    return true;
}

do {
    global $sleepTimeout;
    mailsleep();
    print getLogDate() . "Reading configuration... ";
    $cfg     = new FaxServer();
    $cfg->read("", $configurationId);
    if(!$cfg) {
        print "failed.\n";
        print "ERROR: Could not read configuration.\n";
        continue;
    }
    print "done.\n";


    print getLogDate() . "Connecting to mailbox... ";
    $mailbox    = new InnovaphoneMailbox($cfg->mailHost, $cfg->mailPort, $cfg->mailUser, $cfg->mailPassword, $cfg->mailFolder);
    if(!$mailbox->isConnected()) {
        print "ERROR: Could not connect to the mailbox.\n";
        print "Error message:\n" . $mailbox->getErrors(). "\n";
        continue;
    }
    print "done.\n";


    print getLogDate() . "Attaching to shared memory... ";
    $faxShm     = new InnovaphoneFaxShm($cfg->shmId);
    if(!$faxShm->isAttached()) {
        print "ERROR: Could not attach to the SHM.\n";
        continue;
    }
    print "done.\n";


    print getLogDate() . "Connecting to database... ";
    $faxDatabase    = new InnovaphoneFaxDatabase();
    if(!$faxDatabase->isConnected()) {
        print "ERROR: Could not connect to the database.\n";
        continue;
    }
    $faxDatabase->setServerId($cfg->id);
    print "done.\n";

    print "\n";
    
    $callsDynLimit = 3;

    do {
        if(!$mailbox->isConnected()) {
            if(!$mailbox->connect()) {
                break;
            }
        }

        if($faxShm->read() === false) break;
        $startMax = $callsDynLimit - count($faxShm->getCalls());
        if(startOutgoingCalls($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId, $startMax) === false) break;
        if(doOutgoingConfirm($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId) === false) break;
        if(doIncomingConvert($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId) === false) break;
        if(doIncomingForward($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId) === false) break;
        if(($count = doOutgoingConvert($cfg, $mailbox, $faxDatabase, $faxShm, $configurationId, $startMax)) === false) break;
        
        if($faxShm->read() === false) break;
        if(count($faxShm->getCalls()) + 2 > $callsDynLimit) {
            $newLimit = 2;
            foreach($faxShm->getCalls() as $faxCall) {
                switch($faxCall->getFaxState()) {
                    case('connecting'):
                    case('connected'):
                    case('transmitting'):
                    case('receiving'):
                    case('completed'):
                        $newLimit++;
                }
            }
            if($newLimit > $callsDynLimit) {
                $callsDynLimit = $newLimit;
                print getLogDate() . "New call limit " . $newLimit . ".\n";
                continue;
            }
        }
            
        $sleepTimeout = 0;      // reset sleep timeout after one successfull run
        if($count) {
            sleep(5);
        }
        else {
            $mailbox->disconnect();
            sleep(30);
        }
        
    } while(true);

    sleep(3);
    
} while(true);

?>
